#include "amt/raycaster.hpp"
#include <iostream>
#include <chrono>
#include <numeric>
#include <functional>

#define RAY_VIEW_WIDTH 960
#define RAY_VIEW_HEIGHT 720
#define RAY_VIEW_X (1280 - RAY_VIEW_WIDTH)
#define RAY_VIEW_Y 0

static const int SCREEN_HEIGHT=720;
static const int SCREEN_WIDTH=1280;

Matrix MAP{
  {1,1,1,1,1,1,1,1,1},
  {1,0,2,0,0,0,0,0,1},
  {1,0,4,0,0,5,2,0,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,0,0,0,0,0,1,1},
  {1,0,0,1,3,4,0,0,1},
  {1,0,0,0,0,0,1,1,1},
  {1,0,0,0,0,0,0,0,1},
  {1,1,1,1,1,1,1,1,1}
};

void draw_gui(sf::RenderWindow &window) {
  sf::RectangleShape rect({SCREEN_WIDTH - RAY_VIEW_WIDTH, 300});
  rect.setPosition({0,0});
  rect.setFillColor({100, 100, 100});
  window.draw(rect);
}

int main() {
  sf::RenderWindow window(sf::VideoMode({SCREEN_WIDTH, SCREEN_HEIGHT}), "Zed's Ray Caster Game Thing");

  //ZED this should set with a function
  float player_x = MAP.rows() / 2;
  float player_y = MAP.cols() / 2;

  Raycaster rayview(window, MAP, RAY_VIEW_WIDTH, RAY_VIEW_HEIGHT);
  rayview.set_position(RAY_VIEW_X, RAY_VIEW_Y);
  rayview.position_camera(player_x, player_y);

  double moveSpeed = 0.1;
  double rotSpeed = 0.1;

  const auto onClose = [&window](const sf::Event::Closed&)
  {
      window.close();
  };


  std::size_t const max_count = 100;
  std::vector<double> frames(max_count);
  std::size_t it = 1;
  while(window.isOpen()) {
    auto start = std::chrono::high_resolution_clock::now();
    rayview.render();
    auto end = std::chrono::high_resolution_clock::now();
    auto elapsed = std::chrono::duration<double>(end - start);
    auto frame = 1 / elapsed.count();
    frames.push_back(frame);
    if (it % max_count == 0) {
      auto frame = std::accumulate(frames.begin(), frames.end(), 0., std::plus<>{}) / max_count;
      std::cout << "Frame: " << frame << '\n';
        frames.clear();
        it = 1;
    }
    ++it;

    draw_gui(window);
    window.display();

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::W)) {
      rayview.run(moveSpeed, 1);
    } else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::S)) {
      rayview.run(moveSpeed, -1);
    }

    if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::D)) {
      rayview.rotate(rotSpeed, -1);
    } else if(sf::Keyboard::isKeyPressed(sf::Keyboard::Key::A)) {
      rayview.rotate(rotSpeed, 1);
    }

    window.handleEvents(onClose);
  }

  return 0;
}
